# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2022, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.trackerperspective module."""
# pylint: disable=protected-access
from unittest import skipUnless

from gi.repository import GES

from pitivi.check import MISSING_SOFT_DEPS
from pitivi.trackerperspective import ObjectManager
from tests import common


class TestCoverObjectPopover(common.TestCase):
    """Tests for the CoverObjectPopover class."""

    @skipUnless("cvtracker" not in MISSING_SOFT_DEPS, "cvtracker element missing")
    @common.setup_project_with_clips(assets_names=["tears_of_steel.webm"])
    @common.setup_clipproperties
    def test_cover(self):
        clip, = self.layer.get_clips()
        self.click_clip(clip, expect_selected=True)

        expander = self.clipproperties.effect_expander
        expander.cover_object_button.clicked()
        self.assertTrue(expander.cover_popover.props.visible)
        # Only one row containing the Track Objects button should exist.
        self.assertEqual(len(expander.cover_popover.listbox.get_children()), 1)

        expander.cover_object_button.clicked()
        self.assertFalse(expander.cover_popover.props.visible)

        object_manager = ObjectManager(clip.asset)
        object_manager.add_object(1, "object1", "Object 1")
        object_manager.update_object_position("object1", 0, (10, 20, 30, 40))
        object_manager.add_object(2, "object2", "Object 2")
        object_manager.update_object_position("object2", 1, (20, 30, 40, 50))
        object_manager.save()

        expander.cover_object_button.clicked()
        self.assertTrue(expander.cover_popover.props.visible)
        # Two rows for two object and one for Track Objects.
        self.assertEqual(len(expander.cover_popover.listbox.get_children()), 3)

        self.assertEqual(len(clip.get_top_effects()), 0)
        expander.cover_popover.listbox.get_row_at_index(0).emit("activate")
        self.assertFalse(expander.cover_popover.props.visible)
        self.assertEqual(len(clip.get_top_effects()), 1)

        expander.cover_object_button.clicked()
        self.assertTrue(expander.cover_popover.props.visible)
        # One row for the uncovered object and one for the Track Objects button.
        self.assertEqual(len(expander.cover_popover.listbox.get_children()), 2)


class TestObjectManager(common.TestCase):
    """Tests for the ObjectManager class."""

    def test_load_save(self):
        asset = GES.UriClipAsset.request_sync(common.get_sample_uri("tears_of_steel.webm"))
        object_manager1 = ObjectManager(asset)
        object_manager1.add_object(1, "object1", "Object 1")
        object_manager1.update_object_position("object1", 100, (10, 20, 30, 40))
        object_manager1.save()

        object_manager2 = ObjectManager(asset)
        self.assertListEqual(object_manager2.objects, [(1, "object1", "Object 1")])
        self.assertDictEqual(object_manager2.values, {"object1": [(100, (10, 20, 30, 40))]})

        object_manager2.add_object(2, "object2", "Object 2")
        object_manager2.update_object_position("object2", 200, (20, 30, 40, 50))
        object_manager2.save()

        object_manager3 = ObjectManager(asset)
        self.assertListEqual(object_manager3.objects, [(1, "object1", "Object 1"), (2, "object2", "Object 2")])
        self.assertDictEqual(object_manager2.values, {"object1": [(100, (10, 20, 30, 40))],
                                                      "object2": [(200, (20, 30, 40, 50))]})

    def test_update_object_position(self):
        asset = GES.UriClipAsset.request_sync(common.get_sample_uri("tears_of_steel.webm"))
        object_manager = ObjectManager(asset)
        object_manager.add_object(1, "object1", "Object 1")
        object_manager.update_object_position("object1", 200, (20, 30, 40, 50))
        object_manager.update_object_position("object1", 100, (10, 20, 30, 40))
        object_manager.update_object_position("object1", 300, (30, 40, 50, 60))

        self.assertDictEqual(object_manager.values, {"object1": [(100, (10, 20, 30, 40)),
                                                                 (200, (20, 30, 40, 50)),
                                                                 (300, (30, 40, 50, 60))]})

    def test_interpolate(self):
        asset = GES.UriClipAsset.request_sync(common.get_sample_uri("tears_of_steel.webm"))
        object_manager = ObjectManager(asset)
        object_manager.add_object(1, "object1", "Object 1")
        object_manager.update_object_position("object1", 200, (20, 30, 40, 50))
        object_manager.update_object_position("object1", 100, (10, 20, 30, 40))
        object_manager.update_object_position("object1", 300, (30, 40, 50, 60))

        self.assertTupleEqual(object_manager.interpolate("object1", 99), (10, 20, 30, 40))
        self.assertTupleEqual(object_manager.interpolate("object1", 100), (10, 20, 30, 40))
        self.assertTupleEqual(object_manager.interpolate("object1", 150), (15, 25, 35, 45))
        self.assertTupleEqual(object_manager.interpolate("object1", 200), (20, 30, 40, 50))
        self.assertTupleEqual(object_manager.interpolate("object1", 275), (27.5, 37.5, 47.5, 57.5))
        self.assertTupleEqual(object_manager.interpolate("object1", 300), (30, 40, 50, 60))
        self.assertTupleEqual(object_manager.interpolate("object1", 301), (30, 40, 50, 60))



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2021, Piotr Brzeziński <thewildtreee@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
from gi.repository import GES
from gi.repository import Gst

from pitivi.timeline.markers import ClipMarkersBox
from pitivi.utils.markers import DEFAULT_LIST_KEY
from tests import common


class TestMarkerListManager(common.TestCase):
    def add_single_clip(self):
        clip = GES.TitleClip()
        clip.set_start(5 * Gst.SECOND)
        clip.set_duration(20 * Gst.SECOND)
        self.layer.add_clip(clip)
        return clip

    @common.setup_timeline
    def test_manager_created_with_default(self):
        clip = self.add_single_clip()
        source, = clip.get_children(False)

        manager = source.markers_manager
        self.assertTrue(manager)

        self.assertTrue(manager.list_exists(DEFAULT_LIST_KEY))
        self.assertEqual(manager.current_list_key, DEFAULT_LIST_KEY)

    @common.setup_timeline
    def test_manager_list_add(self):
        clip = self.add_single_clip()
        source, = clip.get_children(False)
        manager = source.markers_manager

        self.assertRaises(ValueError, manager.add_list, DEFAULT_LIST_KEY)
        self.assertRaises(ValueError, manager.add_list, "key with spaces")
        self.assertRaises(ValueError, manager.add_list, None)

        test_key = "test_list"
        markers = [1, 5, 10]
        test_list = manager.add_list(test_key, markers)

        self.assertTrue(manager.list_exists(test_key))
        self.assertEqual(test_list, source.get_marker_list(test_key))
        self.assert_markers(test_list, [(pos, None) for pos in markers])

    @common.setup_timeline
    def test_manager_list_remove(self):
        clip = self.add_single_clip()
        source, = clip.get_children(False)
        manager = source.markers_manager

        self.assertRaises(ValueError, manager.remove_list, DEFAULT_LIST_KEY)
        self.assertRaises(ValueError, manager.remove_list, None)

        test_key = "test_list"
        manager.add_list(test_key)
        manager.current_list_key = test_key

        self.assertEqual(manager.current_list_key, test_key)
        manager.remove_list(test_key)
        self.assertEqual(manager.current_list_key, DEFAULT_LIST_KEY)
        self.assertIsNone(source.get_marker_list(test_key))

    @common.setup_timeline
    def test_manager_default_snappability(self):
        clip = self.add_single_clip()
        source, = clip.get_children(False)
        manager = source.markers_manager

        test_key = "test_list"
        default_snappable = self.app.settings.markersSnappableByDefault

        manager.add_list(test_key)
        manager.current_list_key = test_key
        self.assertEqual(manager.snappable, default_snappable)

    @common.setup_timeline
    def test_manager_current_list(self):
        clip = self.add_single_clip()
        source, = clip.get_children(False)
        manager = source.markers_manager

        test_key = "test_list"
        manager.add_list(test_key)

        # Toggle snappability on the default list, switch to a diff. one,
        # turn off snappability there, and test if they're both
        # correctly kept between active list changes.
        manager.snappable = True
        manager.current_list_key = test_key
        manager.snappable = False

        manager.current_list_key = DEFAULT_LIST_KEY
        self.assertEqual(manager.current_list, source.get_marker_list(DEFAULT_LIST_KEY))
        self.assertTrue(manager.snappable)
        manager.current_list_key = test_key
        self.assertEqual(manager.current_list, source.get_marker_list(test_key))
        self.assertFalse(manager.snappable)
        manager.current_list_key = ""
        self.assertIsNone(manager.current_list)
        self.assertFalse(manager.snappable)
        manager.current_list_key = DEFAULT_LIST_KEY
        self.assertEqual(manager.current_list, source.get_marker_list(DEFAULT_LIST_KEY))
        self.assertTrue(manager.snappable)

    @common.setup_timeline
    def test_manager_marker_box(self):
        clip = self.add_single_clip()
        source, = clip.get_children(False)
        manager = source.markers_manager
        box = ClipMarkersBox(self.app, source)

        test_key = "test_list"
        manager.add_list(test_key)

        self.assertIsNone(box.markers_container)
        manager.set_markers_box(box)
        self.assertEqual(box.markers_container, manager.current_list)
        manager.current_list_key = test_key
        self.assertEqual(box.markers_container, manager.current_list)
        manager.current_list_key = ""
        self.assertIsNone(box.markers_container)

        # Disconnect the box, make a few changes and attach again.
        manager.set_markers_box(None)
        manager.current_list_key = DEFAULT_LIST_KEY
        self.assertIsNone(box.markers_container)
        manager.current_list_key = test_key
        manager.set_markers_box(box)
        self.assertEqual(box.markers_container, manager.current_list)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2021, Tyler Senne <tsenne2@huskers.unl.edu>
# Copyright (c) 2021, Michael Ervin <michael.ervin@huskers.unl.edu>
# Copyright (c) 2021, Aaron Friesen <afriesen4@huskers.unl.edu>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.clipproperties module."""
# pylint: disable=protected-access,no-self-use,import-outside-toplevel,no-member
from gi.repository import Gst

from tests import common


class CompositingPropertiesTest(common.TestCase):

    @common.setup_project_with_clips(assets_names=["1sec_simpsons_trailer.mp4"])
    @common.setup_clipproperties
    def test_max_fade_duration(self):
        fade_in_adjustment = self.compositing_box._fade_in_adjustment
        fade_out_adjustment = self.compositing_box._fade_out_adjustment

        clip, = self.layer.get_clips()
        self.timeline_container.timeline.selection.select([clip])

        self.assertEqual(fade_in_adjustment.props.upper * Gst.SECOND, clip.duration)
        self.assertEqual(fade_out_adjustment.props.upper * Gst.SECOND, clip.duration)

        fade_in_adjustment.props.value = 0.5
        self.assertEqual(fade_out_adjustment.props.upper * Gst.SECOND, clip.duration - (0.5 * Gst.SECOND))

        fade_out_adjustment.props.value = 0.3
        self.assertEqual(fade_in_adjustment.props.upper * Gst.SECOND, clip.duration - (0.3 * Gst.SECOND))

    def _get_control_source(self, clip):
        source = self.get_clip_element(clip)
        control_binding = source.get_control_binding("alpha")
        self.assertIsNotNone(control_binding)
        control_source = control_binding.props.control_source
        self.assertIsNotNone(control_source)
        return control_source

    @common.setup_project_with_clips(assets_names=["1sec_simpsons_trailer.mp4"])
    @common.setup_clipproperties
    def test_apply_keyframes(self):
        clip, = self.layer.get_clips()
        self.timeline_container.timeline.selection.select([clip])

        fade_in_adjustment = self.compositing_box._fade_in_adjustment
        fade_out_adjustment = self.compositing_box._fade_out_adjustment
        fade_in_adjustment.props.value = 0.5
        fade_out_adjustment.props.value = 0.3

        control_source = self._get_control_source(clip)
        self.assert_control_source_values(control_source,
                                          [0, 1, 1, 0],
                                          [0, 0.5 * Gst.SECOND, clip.duration - 0.3 * Gst.SECOND, clip.duration])

    @common.setup_project_with_clips(assets_names=["1sec_simpsons_trailer.mp4"])
    @common.setup_clipproperties
    def test_move_keyframes(self):
        clip, = self.layer.get_clips()
        self.timeline_container.timeline.selection.select([clip])

        control_source = self._get_control_source(clip)
        control_source.set(0, 0)  # Necessary in order for fade-in to be recognized
        control_source.set(clip.duration, 0)  # Necessary in order for fade-out to be recognized
        control_source.set(0.6 * Gst.SECOND, 0.5)  # This keyframe should be unaffected
        control_source.set(0.1 * Gst.SECOND, 1)  # This keyframe should be the fade-in
        control_source.set(clip.duration - (0.2 * Gst.SECOND), 1)  # This keyframe should be the fade-out

        fade_in_adjustment = self.compositing_box._fade_in_adjustment
        fade_out_adjustment = self.compositing_box._fade_out_adjustment
        self.assertEqual(fade_in_adjustment.props.value, 0.1)
        self.assertEqual(fade_out_adjustment.props.value, 0.2)

        fade_in_adjustment.props.value = 0.5
        fade_out_adjustment.props.value = 0.3

        self.assert_control_source_values(control_source,
                                          [0, 1, 0.5, 1, 0],
                                          [0, 0.5 * Gst.SECOND, 0.6 * Gst.SECOND, clip.duration - 0.3 * Gst.SECOND, clip.duration])

    @common.setup_project_with_clips(assets_names=["1sec_simpsons_trailer.mp4", "30fps_numeroted_frames_blue.webm"])
    @common.setup_clipproperties
    def test_adjustments_updated_when_switching_clips(self):
        clip1, clip2 = self.layer.get_clips()
        self.timeline_container.timeline.selection.select([clip2])

        fade_in_adjustment = self.compositing_box._fade_in_adjustment
        fade_out_adjustment = self.compositing_box._fade_out_adjustment
        self.assertEqual(fade_in_adjustment.props.upper * Gst.SECOND, clip2.duration)
        self.assertEqual(fade_out_adjustment.props.upper * Gst.SECOND, clip2.duration)

        fade_in_adjustment.props.value = 1.3
        mainloop = common.create_main_loop()
        mainloop.run(until_empty=True)
        self.assertEqual(int(fade_out_adjustment.props.upper * Gst.SECOND), clip2.duration - int(1.3 * Gst.SECOND))

        fade_out_adjustment.props.value = 0.5
        self.assertEqual(int(fade_in_adjustment.props.upper * Gst.SECOND), clip2.duration - int(0.5 * Gst.SECOND))

        self.timeline_container.timeline.selection.select([clip1])
        self.assertEqual(fade_in_adjustment.props.value, 0)
        self.assertEqual(fade_out_adjustment.props.value, 0)
        self.assertEqual(fade_in_adjustment.props.upper * Gst.SECOND, clip1.duration)
        self.assertEqual(fade_out_adjustment.props.upper * Gst.SECOND, clip1.duration)

    @common.setup_project_with_clips(assets_names=["1sec_simpsons_trailer.mp4"])
    @common.setup_clipproperties
    def test_adjustments_updated_when_keyframes_updated(self):
        clip, = self.layer.get_clips()
        self.timeline_container.timeline.selection.select([clip])

        fade_in_adjustment = self.compositing_box._fade_in_adjustment
        fade_out_adjustment = self.compositing_box._fade_out_adjustment
        fade_in_adjustment.props.value = 0.5
        fade_out_adjustment.props.value = 0.3

        # Move the keyframes
        control_source = self._get_control_source(clip)
        control_source.unset(0.5 * Gst.SECOND)
        control_source.set(0.4 * Gst.SECOND, 1)
        control_source.unset(clip.duration - (0.3 * Gst.SECOND))
        control_source.set(clip.duration - (0.2 * Gst.SECOND), 1)

        self.assertEqual(fade_in_adjustment.props.value, 0.4)
        self.assertEqual(fade_out_adjustment.props.value, 0.2)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2015, Thibault Saunier <tsaunier@gnome.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the timeline.previewers module."""
# pylint: disable=protected-access,unused-argument
import functools
import os
import tempfile
from unittest import mock

import numpy
from gi.repository import Gdk
from gi.repository import GdkPixbuf
from gi.repository import GES
from gi.repository import Gst

from pitivi.timeline.previewers import delete_all_files_in_dir
from pitivi.timeline.previewers import get_wavefile_location_for_uri
from pitivi.timeline.previewers import Previewer
from pitivi.timeline.previewers import THUMB_HEIGHT
from pitivi.timeline.previewers import THUMB_PERIOD
from pitivi.timeline.previewers import ThumbnailCache
from pitivi.utils.timeline import EditingContext
from pitivi.utils.timeline import Zoomable
from tests import common
from tests.test_medialibrary import BaseTestMediaLibrary


SIMPSON_WAVFORM_VALUES = [
    0.10277689604421922, 0.5078891671078481, 0.913001438171477, 1.318113709235106,
    1.7232259802987349, 2.1283382513623637, 2.5334505224259924, 2.9385627934896212,
    3.34367506455325, 3.748787335616879, 3.7487873356168793, 3.697464549118655,
    3.646141762620431, 3.5948189761222067, 3.5434961896239825, 3.4921734031257583,
    3.440850616627534, 3.38952783012931, 3.3382050436310857, 3.2868822571328615,
    3.2868822571328606, 3.4364524655420996, 3.5860226739513386, 3.7355928823605775,
    3.8851630907698165, 4.034733299179055, 4.1843035075882939, 4.3338737159975329,
    4.4834439244067719, 4.6330141328160108, 4.6330141328160108, 4.6166031969548991,
    4.6001922610937873, 4.5837813252326756, 4.5673703893715638, 4.5509594535104521,
    4.5345485176493403, 4.5181375817882286, 4.5017266459271168, 4.485315710066005,
    4.4853157100660033, 4.5547060070202106, 4.6240963039744178, 4.6934866009286251,
    4.7628768978828324, 4.8322671948370397, 4.901657491791247, 4.9710477887454543,
    5.0404380856996616, 5.1098283826538689, 5.109828382653868, 5.0866990072166436,
    5.0635696317794192, 5.0404402563421948, 5.0173108809049705, 4.9941815054677461,
    4.9710521300305217, 4.9479227545932973, 4.924793379156073, 4.9016640037188486,
    4.901664003718845, 4.8508680589687616, 4.8000721142186782, 4.7492761694685948,
    4.6984802247185113, 4.6476842799684279, 4.5968883352183445, 4.5460923904682611,
    4.4952964457181777, 4.4445005009680942, 4.4445005009680925, 4.5544001248210364,
    4.6642997486739803, 4.7741993725269243, 4.8840989963798682, 4.9939986202328122,
    5.1038982440857561, 5.2137978679387, 5.323697491791644, 5.4335971156445879,
    5.433597115644587, 5.3988578703107724, 5.3641186249769577, 5.329379379643143,
    5.2946401343093283, 5.2599008889755137, 5.225161643641699, 5.1904223983078843,
    5.1556831529740696, 5.120943907640255, 5.120943907640255, 5.0895954243424724,
    5.0582469410446897, 5.0268984577469071, 4.9955499744491245, 4.9642014911513419,
    4.9328530078535593, 4.9015045245557767, 4.8701560412579941, 4.8388075579602114,
    4.8388075579602079, 4.7336937056290518, 4.6285798532978957, 4.5234660009667396,
    4.4183521486355835, 4.3132382963044273, 4.2081244439732712, 4.1030105916421151,
    3.9978967393109586, 3.892782886979802, 3.8927828869797994, 3.8630968485705619,
    3.8334108101613245, 3.803724771752087, 3.7740387333428496, 3.7443526949336121,
    3.7146666565243747, 3.6849806181151372, 3.6552945797058998, 3.6256085412966623,
    3.6256085412966614, 0.0]


class TestPreviewers(BaseTestMediaLibrary):

    def setup_trimmed_clips(self, frame_count):
        # Set the project videorate to match tears_of_steel.webm.
        self.project.videorate = Gst.Fraction(24, 1)
        Zoomable.set_zoom_level(78)

        timeline = self.timeline_container.timeline

        start_frame = 43
        start = self.timeline.get_frame_time(start_frame)
        self.assertEqual(start, 1791666667)

        clips = []
        starts = []
        asset = GES.UriClipAsset.request_sync(common.get_sample_uri("tears_of_steel.webm"))
        for delta_frame in range(frame_count):
            ges_layer = timeline.ges_timeline.append_layer()
            ges_clip = timeline.add_clip_to_layer(ges_layer, asset, start)

            editing_context = EditingContext(ges_clip, self.timeline, GES.EditMode.EDIT_TRIM, GES.Edge.EDGE_START, self.app)
            new_start = self.timeline.get_frame_time(start_frame + delta_frame)
            editing_context.edit_to(new_start, ges_layer)
            editing_context.finish()

            clips.append(ges_clip)
            starts.append(new_start)

        expected_starts = [1791666667, 1833333334, 1875000000, 1916666667, 1958333334, 2000000000, 2041666667, 2083333334, 2125000000, 2166666667,
                           2208333334, 2250000000, 2291666667, 2333333334, 2375000000, 2416666667, 2458333334, 2500000000, 2541666667, 2583333334,
                           2625000000, 2666666667, 2708333334, 2750000000, 2791666667, 2833333334, 2875000000, 2916666667, 2958333334, 3000000000,
                           3041666667, 3083333334, 3125000000, 3166666667, 3208333334, 3250000000, 3291666667, 3333333334, 3375000000, 3416666667,
                           3458333334, 3500000000, 3541666667, 3583333334, 3625000000, 3666666667, 3708333334, 3750000000][:frame_count]

        # Check we calculated correctly above where the clips should be placed.
        self.assertListEqual(starts, expected_starts)

        # Check the clips ended up exactly where we placed them.
        self.assertListEqual([c.start for c in clips], expected_starts,
                             "the clips have been placed in unexpected positions")

        expected_inpoints = [0, 41666667, 83333333, 125000000, 166666667, 208333333, 250000000, 291666667, 333333333, 375000000,
                             416666667, 458333333, 500000000, 541666667, 583333333, 625000000, 666666667, 708333333, 750000000, 791666667,
                             833333333, 875000000, 916666667, 958333333, 1000000000, 1041666667, 1083333333, 1125000000, 1166666667, 1208333333,
                             1250000000, 1291666667, 1333333333, 1375000000, 1416666667, 1458333333, 1500000000, 1541666667, 1583333333, 1625000000,
                             1666666667, 1708333333, 1750000000, 1791666667, 1833333333, 1875000000, 1916666667, 1958333333][:frame_count]
        self.assertListEqual([c.inpoint for c in clips], expected_inpoints)

        return clips


class TestVideoPreviewer(TestPreviewers):
    """Tests for the `VideoPreviewer` class."""

    @common.setup_timeline
    def test_thumbnails_position_quantization(self):
        clips = self.setup_trimmed_clips(48)

        # Check the video previewers.
        video_previewers = list(self.get_clip_element(c, GES.VideoSource).ui.previewer
                                for c in clips)

        mainloop = common.create_main_loop()

        # Acrobatics to run the mainloop until the pipelines of the previewers
        # are ready.
        updated_previewers = set()

        def update_thumbnails_func(previewer, original):
            nonlocal updated_previewers
            updated_previewers.add(previewer)

            # Restore the original method.
            previewer._update_thumbnails = original

            if set(updated_previewers) == set(video_previewers):
                mainloop.quit()

        for previewer in video_previewers:
            original = previewer._update_thumbnails
            previewer._update_thumbnails = functools.partial(update_thumbnails_func, previewer, original)

        mainloop.run()

        expected_queues = (
            [[0, 500000000, 1000000000, 1500000000]] * 12 +
            [[500000000, 1000000000, 1500000000]] * 12 +
            [[1000000000, 1500000000]] * 12 +
            [[1500000000]] * 12
        )

        expected_x_positions = (
            [[0, 237, 474, 712]] * 12 +
            [[0, 237, 474]] * 12 +
            [[0, 237]] * 12 +
            [[0]] * 12
        )

        for previewer, expected_queue, expected_xs in zip(video_previewers, expected_queues, expected_x_positions):
            self.assertEqual(previewer.thumb_width, 151)

            xs = []

            def move_put_func(thumb, x, y):
                # pylint: disable=cell-var-from-loop
                xs.append(x)

            previewer.move = move_put_func
            previewer.put = move_put_func
            try:
                previewer.refresh()
            finally:
                # These should not be called anymore.
                previewer.move = mock.Mock()
                previewer.put = mock.Mock()

            self.assertListEqual(previewer.queue, expected_queue)
            self.assertListEqual(xs, expected_xs)


class TestAudioPreviewer(TestPreviewers):
    """Tests for the `AudioPreviewer` class."""

    def test_create_thumbnail_bin(self):
        """Checks our `waveformbin` element is usable."""
        pipeline = Gst.parse_launch("uridecodebin name=decode uri=file:///some/thing"
                                    " waveformbin name=wavebin ! fakesink qos=false name=faked")
        self.assertTrue(pipeline)
        wavebin = pipeline.get_by_name("wavebin")
        self.assertTrue(wavebin)

    def test_waveform_creation(self):
        """Checks the waveform generation."""
        sample_name = "1sec_simpsons_trailer.mp4"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name])

            sample_uri = common.get_sample_uri(sample_name)
            wavefile = get_wavefile_location_for_uri(sample_uri)

        self.assertTrue(os.path.exists(wavefile), wavefile)

        with open(wavefile, "rb") as fsamples:
            samples = list(numpy.load(fsamples))

        self.assertEqual(samples, SIMPSON_WAVFORM_VALUES)

    @common.setup_timeline
    def test_waveform_offset(self):
        clips = self.setup_trimmed_clips(10)

        # Check the audio previewers.
        audio_previewers = list(self.get_clip_element(c, GES.AudioSource).ui.previewer
                                for c in clips)

        offsets = []

        def set_source_surface(surface, offset_x, offset_y):
            offsets.append(offset_x)

        samples = list(range(199))
        for previewer in audio_previewers:
            previewer.samples = samples
            with mock.patch.object(Gdk, "cairo_get_clip_rectangle") as cairo_get_clip_rectangle:
                cairo_get_clip_rectangle.return_value = (True, mock.Mock(x=0, width=10000))
                from pitivi.timeline import previewers
                with mock.patch.object(previewers.renderer, "fill_surface") as fill_surface:
                    context = mock.Mock()
                    context.set_source_surface = set_source_surface
                    previewer.do_draw(context)
            fill_surface.assert_called_once_with(samples, 949, -1)

        expected_offsets = [0, -20, -40, -59, -79, -99, -119, -138, -158, -178]
        self.assertListEqual(offsets, expected_offsets)


class TestPreviewer(common.TestCase):
    """Tests for the `Previewer` class."""

    def test_thumb_interval(self):
        """Checks the `thumb_interval` method."""
        def run_thumb_interval(interval):
            """Runs thumb_interval."""
            with mock.patch("pitivi.utils.timeline.Zoomable.pixel_to_ns") as pixel_to_ns:
                pixel_to_ns.return_value = interval
                return Previewer.thumb_interval(1)

        self.assertEqual(run_thumb_interval(1), THUMB_PERIOD)
        self.assertEqual(run_thumb_interval(THUMB_PERIOD - 1), THUMB_PERIOD)
        self.assertEqual(run_thumb_interval(THUMB_PERIOD), THUMB_PERIOD)

        self.assertEqual(run_thumb_interval(THUMB_PERIOD + 1), 2 * THUMB_PERIOD)
        self.assertEqual(run_thumb_interval(2 * THUMB_PERIOD - 1), 2 * THUMB_PERIOD)
        self.assertEqual(run_thumb_interval(2 * THUMB_PERIOD), 2 * THUMB_PERIOD)


class TestThumbnailCache(BaseTestMediaLibrary):
    """Tests for the ThumbnailCache class."""

    def test_get(self):
        """Checks the `get` method returns the same thing for asset and URI."""
        with self.assertRaises(ValueError):
            ThumbnailCache.get(1)
        with mock.patch("pitivi.timeline.previewers.xdg_cache_home") as xdg_config_home,\
                tempfile.TemporaryDirectory() as temp_dir:
            xdg_config_home.return_value = temp_dir
            sample_uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
            cache = ThumbnailCache.get(sample_uri)
            self.assertIsNotNone(cache)

            asset = GES.UriClipAsset.request_sync(sample_uri)
            self.assertEqual(ThumbnailCache.get(asset), cache)

    def test_image_size(self):
        """Checks the `image_size` property."""
        with tempfile.TemporaryDirectory() as tmpdirname:
            with mock.patch("pitivi.timeline.previewers.xdg_cache_home") as xdg_cache_home:
                xdg_cache_home.return_value = tmpdirname
                sample_uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
                thumb_cache = ThumbnailCache(sample_uri)
                self.assertEqual(thumb_cache.image_size, (0, 0))

                pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB,
                                              False, 8, 20, 10)
                thumb_cache[0] = pixbuf
                self.assertEqual(thumb_cache.image_size, (20, 10))

    def test_containment(self):
        """Checks the __contains/getitem/setitem__ methods."""
        with tempfile.TemporaryDirectory() as tmpdirname:
            with mock.patch("pitivi.timeline.previewers.xdg_cache_home") as xdg_cache_home:
                xdg_cache_home.return_value = tmpdirname
                sample_uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
                thumb_cache = ThumbnailCache(sample_uri)
                self.assertFalse(Gst.SECOND in thumb_cache)
                with self.assertRaises(KeyError):
                    # pylint: disable=pointless-statement
                    thumb_cache[Gst.SECOND]

                pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB,
                                              False, 8,
                                              int(THUMB_HEIGHT * 1280 / 544), THUMB_HEIGHT)
                thumb_cache[Gst.SECOND] = pixbuf
                self.assertTrue(Gst.SECOND in thumb_cache)
                self.assertIsNotNone(thumb_cache[Gst.SECOND])
                thumb_cache.commit()

                thumb_cache = ThumbnailCache(sample_uri)
                self.assertTrue(Gst.SECOND in thumb_cache)
                self.assertIsNotNone(thumb_cache[Gst.SECOND])


class TestFunctions(BaseTestMediaLibrary):
    """Tests for the standalone functions."""

    def test_delete_all_files_in_dir(self):
        """Checks whether files in sub directories are deleted."""
        with tempfile.TemporaryDirectory() as dir_a, \
                tempfile.NamedTemporaryFile(dir=dir_a, delete=False), \
                tempfile.TemporaryDirectory(dir=dir_a) as dir_a_b, \
                tempfile.NamedTemporaryFile(dir=dir_a_b) as file_a_b1:
            self.assertEqual(len(os.listdir(dir_a)), 2)
            self.assertEqual(len(os.listdir(dir_a_b)), 1)
            delete_all_files_in_dir(dir_a)
            self.assertEqual(os.listdir(dir_a), [os.path.basename(dir_a_b)])
            self.assertEqual(os.listdir(dir_a_b), [os.path.basename(file_a_b1.name)])



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2020, Michael Westburg <michael.westberg@huskers.unl.edu>
# Copyright (c) 2020, Matt Lowe <mattlowe13@huskers.unl.edu>
# Copyright (c) 2020, Aaron Byington <aabyington4@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.peakmeter module."""
from gi.repository import GLib

from pitivi.project import ProjectManager
from pitivi.viewer.peak_meter import MIN_PEAK
from pitivi.viewer.viewer import ViewerContainer
from tests import common


class TestPeakMeter(common.TestCase):
    """Tests for the peak meter."""

    def test_peak_meter_update(self):
        """Checks that the peak value updates correctly when audio is played."""
        app = common.create_pitivi_mock()
        app.project_manager = ProjectManager(app)

        viewer = ViewerContainer(app)
        self.assertListEqual(viewer.peak_meters, [])

        uri = common.get_sample_uri("1sec_simpsons_trailer.mp4")
        xges = r"""<ges version='0.7'>
  <project properties='properties;' metadatas='metadatas, author=(string)Author, scaled_proxy_width=(int)0, scaled_proxy_height=(int)0, render-scale=(double)100, format-version=(string)0.7;'>
    <encoding-profiles>
      <encoding-profile name='pitivi-profile' description='Pitivi encoding profile' type='container' preset-name='webmmux' format='video/webm' >
        <stream-profile parent='pitivi-profile' id='0' type='video' presence='0' format='video/x-vp8, profile=(string){ 0, 1, 2, 3 }' preset-name='vp8enc' restriction='video/x-raw, width=(int)1080, height=(int)1920, framerate=(fraction)30/1, pixel-aspect-ratio=(fraction)1/1' pass='0' variableframerate='0' />
        <stream-profile parent='pitivi-profile' id='1' type='audio' presence='0' format='audio/x-vorbis, rate=(int)[ 1, 200000 ], channels=(int)[ 1, 255 ]' preset-name='vorbisenc' restriction='audio/x-raw, rate=(int)48000, channels=(int)2' />
      </encoding-profile>
    </encoding-profiles>
    <ressources>
      <asset id='%(uri)s' extractable-type-name='GESUriClip' properties='properties, supported-formats=(int)6, duration=(guint64)1228000000;' metadatas='metadatas, video-codec=(string)&quot;H.264\ /\ AVC&quot;, bitrate=(uint)1370124, datetime=(datetime)2007-02-19T05:03:04Z, encoder=(string)Lavf54.6.100, container-format=(string)&quot;ISO\ MP4/M4A&quot;, audio-codec=(string)&quot;MPEG-4\ AAC\ audio&quot;, maximum-bitrate=(uint)130625, file-size=(guint64)232417;' >
        <stream-info id='11c5d3bc5140b4cd95fc0c2b7125dff3b9c6db88183a85200821b61365719f91/002' extractable-type-name='GESAudioUriSource' properties='properties, track-type=(int)2;' metadatas='metadatas;' caps='audio/mpeg, mpegversion=(int)4, framed=(boolean)true, stream-format=(string)raw, level=(string)2, base-profile=(string)lc, profile=(string)lc, codec_data=(buffer)1190, rate=(int)48000, channels=(int)2'/>
        <stream-info id='11c5d3bc5140b4cd95fc0c2b7125dff3b9c6db88183a85200821b61365719f91/001' extractable-type-name='GESVideoUriSource' properties='properties, track-type=(int)4;' metadatas='metadatas;' caps='video/x-h264, stream-format=(string)avc, alignment=(string)au, level=(string)3.1, profile=(string)high, codec_data=(buffer)0164001fffe100176764001facd94050045a1000003e90000bb800f183196001000668ebe3cb22c0, width=(int)1280, height=(int)544, framerate=(fraction)24000/1001, pixel-aspect-ratio=(fraction)1/1, interlace-mode=(string)progressive, chroma-format=(string)4:2:0, bit-depth-luma=(uint)8, bit-depth-chroma=(uint)8, parsed=(boolean)true'/>
      </asset>
      <asset id='crossfade' extractable-type-name='GESTransitionClip' properties='properties, supported-formats=(int)6;' metadatas='metadatas, description=(string)GES_VIDEO_STANDARD_TRANSITION_TYPE_CROSSFADE;' >
      </asset>
    </ressources>
    <timeline properties='properties, auto-transition=(boolean)true, snapping-distance=(guint64)6420600;' metadatas='metadatas, markers=(GESMarkerList)&quot;EMPTY&quot;, duration=(guint64)2456000000;'>
      <track caps='video/x-raw(ANY)' track-type='4' track-id='0' properties='properties, message-forward=(boolean)true, restriction-caps=(string)&quot;video/x-raw\,\ width\=\(int\)1080\,\ height\=\(int\)1920\,\ framerate\=\(fraction\)30/1\,\ pixel-aspect-ratio\=\(fraction\)1/1&quot;, id=(string)87987a781a347bd399437dc56c9c6cd7;' metadatas='metadatas;'/>
      <track caps='audio/x-raw(ANY)' track-type='2' track-id='1' properties='properties, message-forward=(boolean)true, restriction-caps=(string)&quot;audio/x-raw\,\ rate\=\(int\)48000\,\ channels\=\(int\)2&quot;, id=(string)866a7d96e4e467e9f07ac713d07551de;' metadatas='metadatas;'/>
      <layer priority='0' properties='properties, auto-transition=(boolean)true;' metadatas='metadatas, volume=(float)1;'>
        <clip id='0' asset-id='%(uri)s' type-name='GESUriClip' layer-priority='0' track-types='6' start='0' duration='1228000000' inpoint='0' rate='0' properties='properties, name=(string)uriclip1;' metadatas='metadatas;'>
          <source track-id='1' properties='properties, track-type=(int)2, has-internal-source=(boolean)true;'  children-properties='properties, GstVolume::mute=(boolean)false, GstVolume::volume=(double)1;'>
            <binding type='direct' source_type='interpolation' property='volume' mode='1' track_id='1' values =' 0:0.10000000000000001  1228000000:0.10000000000000001 '/>
          </source>
          <source track-id='0' properties='properties, track-type=(int)4, has-internal-source=(boolean)true;'  children-properties='properties, GstFramePositioner::alpha=(double)1, GstDeinterlace::fields=(int)0, GstFramePositioner::height=(int)459, GstDeinterlace::mode=(int)0, GstFramePositioner::posx=(int)0, GstFramePositioner::posy=(int)730, GstDeinterlace::tff=(int)0, GstVideoDirection::video-direction=(int)8, GstFramePositioner::width=(int)1080;'>
            <binding type='direct' source_type='interpolation' property='alpha' mode='1' track_id='0' values =' 0:1  1228000000:1 '/>
          </source>
        </clip>
        <clip id='1' asset-id='%(uri)s' type-name='GESUriClip' layer-priority='0' track-types='6' start='1228000000' duration='1228000000' inpoint='0' rate='0' properties='properties, name=(string)uriclip2;' metadatas='metadatas;'>
          <source track-id='1' properties='properties, track-type=(int)2, has-internal-source=(boolean)true;'  children-properties='properties, GstVolume::mute=(boolean)false, GstVolume::volume=(double)1;'>
            <binding type='direct' source_type='interpolation' property='volume' mode='1' track_id='1' values =' 0:0.10000000000000001  1228000000:0.10000000000000001 '/>
          </source>
          <source track-id='0' properties='properties, track-type=(int)4, has-internal-source=(boolean)true;'  children-properties='properties, GstFramePositioner::alpha=(double)1, GstDeinterlace::fields=(int)0, GstFramePositioner::height=(int)459, GstDeinterlace::mode=(int)0, GstFramePositioner::posx=(int)0, GstFramePositioner::posy=(int)730, GstDeinterlace::tff=(int)0, GstVideoDirection::video-direction=(int)8, GstFramePositioner::width=(int)1080;'>
            <binding type='direct' source_type='interpolation' property='alpha' mode='1' track_id='0' values =' 0:1  1228000000:1 '/>
          </source>
        </clip>
      </layer>
      <groups>
      </groups>
    </timeline>
  </project>
</ges>""" % {"uri": uri}

        proj_uri = self.create_project_file_from_xges(xges)
        project = app.project_manager.load_project(proj_uri)
        # Before the project finished loading it has one audio channel.
        self.assertEqual(project.audiochannels, 1)

        mainloop = common.create_main_loop()

        def project_loaded_cb(project, timeline):
            mainloop.quit()

        project.connect_after("loaded", project_loaded_cb)
        mainloop.run()
        # After the project finishes loading it has the correct number of
        # audio channels.
        self.assertEqual(project.audiochannels, 2)

        self.assertEqual(len(viewer.peak_meters), 0)
        viewer.set_project(project)
        # After setting the project we should have two peak meters.
        self.assertEqual([meter.peak for meter in viewer.peak_meters], [MIN_PEAK, MIN_PEAK])

        # Check that after starting playback we get "peak" values on the bus.
        def bus_message_cb(bus, message):
            if message.get_structure().get_value("peak") is not None:
                mainloop.quit()

        def begin_playback():
            project.pipeline.play()
            project.pipeline.get_bus().connect("message::element", bus_message_cb)

        GLib.idle_add(begin_playback)
        mainloop.run()

        peaks = [meter.peak for meter in viewer.peak_meters]
        for peak in peaks:
            self.assertGreaterEqual(0, peak, peaks)
            self.assertGreaterEqual(peak, MIN_PEAK, peaks)

    @common.setup_project(["tears_of_steel.webm"])
    def test_peak_meter_channels_update(self):
        """Checks that the peak meter channels updates correctly when audio channels changes."""
        viewer = ViewerContainer(self.app)
        viewer.set_project(self.project)

        self.assertEqual(self.project.audiochannels, 1)
        self.assertEqual(len(viewer.peak_meters), 1)

        self.project.audiochannels = 6
        self.assertEqual(len(viewer.peak_meters), 6)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2022, Thejas Kiran P S <thejaskiranps@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
# pylint: disable=protected-access
"""Tests for the pitivi.timeline.ruler module."""
from tests import common


class TestRuler(common.TestCase):
    """Tests for the ScaleRuler class."""

    def test_ruler_scales_sorted(self):
        scales = common.create_timeline_container().ruler._scales

        # Check the scales are sorted correctly.
        self.assertListEqual(sorted(scales), scales)

        # Check the ticks are sorted correctly.
        for unused_interval, ticks in scales:
            self.assertListEqual(sorted(ticks), ticks)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2019, Millan Castro <m.castrovilarino@gmail.com>
# Copyright (c) 2021, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the timeline.markers module."""
from unittest import mock

from gi.repository import Gdk
from gi.repository import GES
from gi.repository import Gtk

from pitivi.utils.timeline import Zoomable
from tests import common


class TestMarkers(common.TestCase):
    """Tests for markers."""

    @common.setup_timeline
    def test_marker_added_ui(self):
        """Checks the add marker UI."""
        markers = self.timeline.get_marker_list("markers")
        marker_box = self.timeline_container.markers

        x = 100
        event = mock.Mock(spec=Gdk.EventButton)
        event.x = x
        event.y = 1
        event.button = Gdk.BUTTON_PRIMARY

        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
            get_event_widget.return_value = marker_box
            event.guiEvent = Gdk.Event.new(Gdk.EventType.BUTTON_PRESS)
            marker_box.do_button_press_event(event)
            event.guiEvent = Gdk.Event.new(Gdk.EventType.BUTTON_RELEASE)
            marker_box.do_button_release_event(event)

        position = Zoomable.pixel_to_ns(event.x)
        self.assert_markers(markers, [(position, None)])

    @common.setup_timeline
    def test_marker_removed_ui(self):
        """Checks the remove marker UI."""
        markers = self.timeline.get_marker_list("markers")
        marker_box = self.timeline_container.markers

        x = 200
        position = Zoomable.pixel_to_ns(x)
        marker = marker_box.markers_container.add(position)
        self.assert_markers(markers, [(position, None)])

        event = mock.Mock(spec=Gdk.EventButton)
        event.x = x
        event.y = 1
        event.button = Gdk.BUTTON_PRIMARY

        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
            get_event_widget.return_value = marker.ui

            def popup(markerpopover):
                # The popover is becoming visible, so we simulate a click.
                markerpopover.remove_button.clicked()

            original_popover_menu = Gtk.Popover.popup
            Gtk.Popover.popup = popup
            try:
                event.type = Gdk.EventType.BUTTON_PRESS
                event.guiEvent = Gdk.Event.new(Gdk.EventType.BUTTON_PRESS)
                marker_box.do_button_press_event(event)

                event.type = Gdk.EventType.DOUBLE_BUTTON_PRESS
                event.guiEvent = Gdk.Event.new(Gdk.EventType.DOUBLE_BUTTON_PRESS)
                marker_box.do_button_press_event(event)
            finally:
                Gtk.Popover.popup = original_popover_menu

        self.assert_markers(markers, [])

    @common.setup_timeline
    def test_marker_moved_ui(self):
        """Checks the move marker UI."""
        markers = self.timeline.get_marker_list("markers")
        marker_box = self.timeline_container.markers

        x1 = 300
        position1 = Zoomable.pixel_to_ns(x1)
        marker = marker_box.markers_container.add(position1)
        self.assert_markers(markers, [(position1, None)])

        x2 = 400
        position2 = Zoomable.pixel_to_ns(x2)

        event = mock.Mock(spec=Gdk.EventButton)
        event.x = x2
        event.y = 1
        event.type = Gdk.EventType.BUTTON_PRESS
        event.button = Gdk.BUTTON_PRIMARY

        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
            get_event_widget.return_value = marker.ui
            event.guiEvent = Gdk.Event.new(Gdk.EventType.BUTTON_PRESS)
            marker_box.do_button_press_event(event)

            with mock.patch.object(marker.ui, "translate_coordinates") as translate_coordinates:
                translate_coordinates.return_value = (x2, 0)
                marker_box.do_motion_notify_event(event)
                marker_box.do_button_release_event(event)

        self.assert_markers(markers, [(position2, None)])

    @common.setup_timeline
    def test_marker_comment_ui(self):
        """Checks the comments marker UI."""
        markers = self.timeline.get_marker_list("markers")
        marker_box = self.timeline_container.markers

        x = 500
        position = Zoomable.pixel_to_ns(x)
        marker = marker_box.markers_container.add(position)
        self.assert_markers(markers, [(position, None)])

        event = mock.Mock(spec=Gdk.EventButton)
        event.x = x
        event.y = 1
        event.button = Gdk.BUTTON_PRIMARY

        with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
            get_event_widget.return_value = marker.ui

            def popup(markerpopover):
                # The popover is becoming visible, so we simulate the user entering text.
                text_buffer = markerpopover.comment_textview.get_buffer()
                text_buffer.set_text("com")
                text_buffer.set_text("comment")
                markerpopover.popdown()

            original_popover_menu = Gtk.Popover.popup
            Gtk.Popover.popup = popup
            try:
                event.type = Gdk.EventType.BUTTON_PRESS
                event.guiEvent = Gdk.Event.new(Gdk.EventType.BUTTON_PRESS)
                marker_box.do_button_press_event(event)

                event.type = Gdk.EventType.DOUBLE_BUTTON_PRESS
                event.guiEvent = Gdk.Event.new(Gdk.EventType.DOUBLE_BUTTON_PRESS)
                marker_box.do_button_press_event(event)
            finally:
                Gtk.Popover.popup = original_popover_menu

        stack, = self.action_log.undo_stacks
        self.assertEqual(len(stack.done_actions), 1, stack.done_actions)

        self.assert_markers(markers, [(position, "comment")])

    def check_seek(self, action, current_position, expected_position):
        pipeline = self.project.pipeline
        with mock.patch.object(pipeline, "get_position") as get_position:
            get_position.return_value = current_position
            with mock.patch.object(pipeline, "simple_seek") as simple_seek:
                action.activate()
                if expected_position is None:
                    simple_seek.assert_not_called()
                else:
                    simple_seek.assert_called_once_with(expected_position)

    @common.setup_timeline
    def test_seeking_actions(self):
        """Checks the seeking actions."""
        # The seek logic ignores the markers past the timeline duration
        # since it's not possible to seek there.
        self.add_clip(self.timeline.layers[0], start=0, duration=20)

        markers = self.timeline.get_marker_list("markers")
        marker_box = self.timeline_container.markers

        marker_box.markers_container.add(10)
        marker_box.markers_container.add(12)
        self.assert_markers(markers, [(10, None), (12, None)])

        self.check_seek(self.timeline_container.seek_forward_marker_action, 9, 10)
        self.check_seek(self.timeline_container.seek_forward_marker_action, 10, 12)
        self.check_seek(self.timeline_container.seek_forward_marker_action, 11, 12)
        self.check_seek(self.timeline_container.seek_forward_marker_action, 12, None)
        self.check_seek(self.timeline_container.seek_forward_marker_action, 13, None)

        self.check_seek(self.timeline_container.seek_backward_marker_action, 9, None)
        self.check_seek(self.timeline_container.seek_backward_marker_action, 10, None)
        self.check_seek(self.timeline_container.seek_backward_marker_action, 11, 10)
        self.check_seek(self.timeline_container.seek_backward_marker_action, 12, 10)
        self.check_seek(self.timeline_container.seek_backward_marker_action, 13, 12)

    @common.setup_timeline
    def test_seeking_with_clips(self):
        """Checks the seeking actions with clip markers present."""
        self.timeline.append_layer()
        timeline = self.timeline_container.timeline
        clip1 = self.add_clip(self.timeline.layers[0], start=0, duration=30)
        clip2 = self.add_clip(self.timeline.layers[1], start=10, duration=20)

        markers1 = next(common.get_clip_children(
            clip1, GES.TrackType.VIDEO)).ui.markers.markers_container
        markers2 = next(common.get_clip_children(
            clip2, GES.TrackType.VIDEO)).ui.markers.markers_container

        markers1.add(5)
        markers1.add(25)
        markers2.add(5)
        markers2.add(15)

        forward_seek = self.timeline_container.seek_forward_marker_action
        backward_seek = self.timeline_container.seek_backward_marker_action

        timeline.selection.select([clip1])
        self.check_seek(forward_seek, 0, 5)
        self.check_seek(forward_seek, 5, 25)
        self.check_seek(forward_seek, 25, None)

        timeline.selection.select([clip2])
        self.check_seek(forward_seek, 25, None)
        self.check_seek(backward_seek, 25, 15)
        self.check_seek(backward_seek, 15, None)

        timeline.selection.select([])
        self.check_seek(forward_seek, 15, None)
        self.check_seek(backward_seek, 15, None)

        # When multiple clips are selected, take all their markers into consideration.
        timeline.selection.select([clip1, clip2])
        self.check_seek(forward_seek, 15, 25)
        self.check_seek(backward_seek, 25, 15)
        self.check_seek(backward_seek, 15, 5)

        # Trim first 10 seconds of clip2, 'cutting off' its first marker.
        clip2.trim(20)
        self.check_seek(forward_seek, 5, 25)

        # Add a marker "outside" clip1 and check if it's correctly ignored.
        markers1.add(50)
        self.check_seek(forward_seek, 25, None)

        # Add a timeline marker which should be ignored while clips are selected.
        timeline_markers = self.timeline_container.markers.markers_container
        timeline_markers.add(28)
        self.check_seek(forward_seek, 25, None)

        # Unselect clips and seek again.
        timeline.selection.select([])
        self.check_seek(forward_seek, 25, 28)

    def perform_at_timeline_position(self, action, position):
        pipeline = self.project.pipeline
        with mock.patch.object(pipeline, "get_position") as get_position:
            get_position.return_value = position
            action.activate()

    @common.setup_timeline
    def test_add_marker_action(self):
        """Checks marker adding shortcut behaviour."""
        self.timeline.append_layer()
        timeline = self.timeline_container.timeline
        add_action = self.timeline_container.add_marker_action
        clip1 = self.add_clip(self.timeline.layers[0], start=0, duration=20)
        clip2 = self.add_clip(self.timeline.layers[1], start=10, duration=20)

        timeline_markers = self.timeline_container.markers.markers_container
        markers1 = next(common.get_clip_children(
            clip1, GES.TrackType.VIDEO)).ui.markers.markers_container
        markers2 = next(common.get_clip_children(
            clip2, GES.TrackType.VIDEO)).ui.markers.markers_container

        # No clips selected - should add a marker to the timeline.
        self.perform_at_timeline_position(add_action, 15)
        self.assert_markers(timeline_markers, [(15, None)])

        # Multiple clips selected - add marker to all of them.
        timeline.selection.select([clip1, clip2])
        self.perform_at_timeline_position(add_action, 15)
        self.assert_markers(markers1, [(15, None)])
        self.assert_markers(markers2, [(5, None)])

        # Adding a marker 'outside' of one clip should fail, but still add to other selected clips.
        self.perform_at_timeline_position(add_action, 5)
        self.assert_markers(markers1, [(5, None), (15, None)])
        self.assert_markers(markers2, [(5, None)])

        self.perform_at_timeline_position(add_action, 25)
        self.assert_markers(markers1, [(5, None), (15, None)])
        self.assert_markers(markers2, [(5, None), (15, None)])

        # Make sure nothing was added to the timeline.
        self.assert_markers(timeline_markers, [(15, None)])

        # One clip selected - make sure no other clips are affected.
        timeline.selection.select([clip1])
        self.perform_at_timeline_position(add_action, 10)
        self.assert_markers(markers1, [(5, None), (10, None), (15, None)])
        self.assert_markers(markers2, [(5, None), (15, None)])

        timeline.selection.select([clip2])
        self.perform_at_timeline_position(add_action, 20)
        self.assert_markers(markers1, [(5, None), (10, None), (15, None)])
        self.assert_markers(markers2, [(5, None), (10, None), (15, None)])

        self.assert_markers(timeline_markers, [(15, None)])



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2021, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.viewer.title_overlay module."""
# pylint: disable=protected-access,no-self-use,attribute-defined-outside-init
from unittest import mock

from gi.repository import Gdk
from gi.repository import GES

from pitivi.viewer.title_overlay import TitleOverlay
from tests import common


class TitleOverlayTest(common.TestCase):
    """Tests for the TitleOverlay class."""

    def assert_title_source_absolute_position(self, source: GES.TitleSource, expected_x: float, expected_y: float):
        res, val = source.get_child_property("x-absolute")
        self.assertTrue(res)
        self.assertAlmostEqual(val, expected_x)
        res, val = source.get_child_property("y-absolute")
        self.assertTrue(res)
        self.assertAlmostEqual(val, expected_y)

    def test_dragging_creates_an_undoable_operation(self):
        mainloop = common.create_main_loop()

        app = common.create_pitivi()
        app.gui = None
        app.create_main_window()
        app.project_manager.new_blank_project()
        mainloop.run(until_empty=True)
        self.assertDictEqual(app.gui.editor.viewer.overlay_stack._OverlayStack__overlays, {})

        app.gui.editor.clipconfig._title_button.clicked()
        mainloop.run(until_empty=True)

        self.assertEqual(len(app.gui.editor.viewer.overlay_stack._OverlayStack__overlays), 1)
        source = app.gui.editor.clipconfig.title_expander.source
        overlay = app.gui.editor.viewer.overlay_stack._OverlayStack__overlays[source]
        self.assertIsInstance(overlay, TitleOverlay)

        overlay_stack = app.gui.editor.viewer.overlay_stack
        # The overlay of the selected clip's video source should have been
        # selected.
        self.assertIs(overlay_stack.selected_overlay, overlay)

        # Simulate hover so it can be dragged.
        w, h = overlay_stack.window_size
        self.assertIs(overlay_stack.hovered_overlay, None)
        event = mock.Mock(type=Gdk.EventType.MOTION_NOTIFY, x=w / 2, y=h / 2)
        overlay_stack.do_event(event)
        self.assertIs(overlay_stack.hovered_overlay, overlay)

        # Simulate drag&drop.
        event = mock.Mock(type=Gdk.EventType.BUTTON_PRESS, x=w / 2, y=h / 2)
        overlay_stack.do_event(event)
        self.assert_title_source_absolute_position(source, 0.5, 0.5)

        event = mock.Mock(type=Gdk.EventType.MOTION_NOTIFY, x=0, y=0)
        overlay_stack.do_event(event)

        event = mock.Mock(type=Gdk.EventType.BUTTON_RELEASE, x=0, y=0)
        overlay_stack.do_event(event)
        mainloop.run(until_empty=True)
        # These values are weird.
        self.assert_title_source_absolute_position(source, -0.24980483996877437, -0.08686210640608036)

        # Undo movement.
        app.action_log.undo()
        self.assert_title_source_absolute_position(source, 0.5, 0.5)
        # Undo title creation.
        app.action_log.undo()

        app.action_log.redo()
        self.assert_title_source_absolute_position(source, 0.5, 0.5)
        app.action_log.redo()
        self.assert_title_source_absolute_position(source, -0.24980483996877437, -0.08686210640608036)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2016, Alex B <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.undo.base module."""
# pylint: disable=protected-access
from unittest import mock

from gi.repository import GES

from pitivi.undo.base import GObjectObserver
from pitivi.undo.base import PropertyChangedAction
from pitivi.undo.undo import UndoableActionLog
from pitivi.undo.undo import UndoableActionStack
from tests import common


class TestGObjectObserver(common.TestCase):
    """Tests for the GObjectObserver class."""

    def test_property_change(self):
        action_log = UndoableActionLog()
        action_log.begin("complex stuff")
        stack = action_log.stacks[0]

        clip = GES.TitleClip()
        clip.props.start = 1
        unused_observer = GObjectObserver(clip, ["start"], action_log)

        self.assertEqual(len(stack.done_actions), 0)
        clip.props.start = 2
        self.assertEqual(len(stack.done_actions), 1)

        clip.props.start = 2
        self.assertEqual(len(stack.done_actions), 1)

        clip.props.start = 4
        self.assertEqual(len(stack.done_actions), 1)
        action = stack.done_actions[-1]
        self.assertEqual(action.old_value, 1)
        self.assertEqual(action.new_value, 4)


class TestPropertyChangedAction(common.TestCase):

    def test_expand(self):
        stack = UndoableActionStack("good one!", mergeable=False)
        gobject = mock.Mock()
        stack.push(PropertyChangedAction(gobject, "field", 5, 7))
        stack.push(PropertyChangedAction(gobject, "field", 11, 13))
        self.assertEqual(len(stack.done_actions), 1, stack.done_actions)
        self.assertEqual(stack.done_actions[0].old_value, 5)
        self.assertEqual(stack.done_actions[0].new_value, 13)

        stack.push(PropertyChangedAction(gobject, "field2", 0, 1))
        self.assertEqual(len(stack.done_actions), 2, stack.done_actions)

        stack.push(PropertyChangedAction(mock.Mock(), "field", 0, 1))
        self.assertEqual(len(stack.done_actions), 3, stack.done_actions)
